/////////////////////////////////////////////////////////////////////////////
// $Header: /cvsroot/mmconvert/mmconvert/MMConvert/Src/Common/DShow.cpp,v 1.5 2005/09/16 08:24:39 ping235 Exp $
//
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 ping235.  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//  This file is subject to the terms of the GNU General Public License as
//  published by the Free Software Foundation.  A copy of this license is
//  included with this software distribution in the file COPYING.  If you
//  do not have a copy, you may obtain a copy by writing to the Free
//  Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  This software is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
// Change Log
//
// Date          Developer             Changes
// $Date: 2005/09/16 08:24:39 $        $Author: ping235 $              $Name:  $
//
/////////////////////////////////////////////////////////////////////////////
// CVS Log
// $Log: DShow.cpp,v $
// Revision 1.5  2005/09/16 08:24:39  ping235
// modified resource
//
// Revision 1.4  2005/09/08 09:43:18  ping235
// translate MMConvert into english
//
// Revision 1.3  2005/08/22 03:38:33  ping235
// modified exception:added context
//
// Revision 1.2  2005/08/10 08:13:14  ping235
// added asf&avi task option function
//
// Revision 1.1.1.1  2005/08/02 05:17:18  ping235
// initial,GUI 1.0.0.0,CMD 1.3.0.1
//
//////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include ".\dshow.h"

CDShow::CDShow(void)
{
}

CDShow::~CDShow(void)
{
}

void CDShow::AddToROT(IGraphBuilder* spGraphBuild,bool bPause,DWORD& dwROTToken)
{
	IRunningObjectTable* spROT = NULL;
	HRESULT hr = GetRunningObjectTable(0,&spROT);
	HANDLE_HR_EXP(hr);

	IMoniker* spMoniker = NULL;
	WCHAR wsz[256];
	IUnknown* spunkGraph = NULL;
	hr = spGraphBuild->QueryInterface(&spunkGraph);
	HANDLE_HR_EXP(hr);

	//static UINT uIndex = 0;
	wsprintfW(wsz,L"FilterGraph %08x pid %08x",(DWORD_PTR)(IUnknown*)spunkGraph,GetCurrentProcessId());
	hr = CreateItemMoniker(L"!",wsz,&spMoniker);
	HANDLE_HR_EXP(hr);
	
	hr = spROT->Register(ROTFLAGS_REGISTRATIONKEEPSALIVE,spunkGraph,spMoniker,&dwROTToken);
	HANDLE_HR_EXP(hr);

	if(bPause)
		AfxMessageBox(_T("You can open GraphEdit to check Filter Graph now, press OK to continue."));
}

void CDShow::RemoveFromROT(DWORD dwROTToken)
{
	if(dwROTToken!=0)
	{
		IRunningObjectTable* spROT = NULL;
		HRESULT hr = GetRunningObjectTable(0,&spROT);
		HANDLE_HR_EXP(hr);

		spROT->Revoke(dwROTToken);
	}
}

HRESULT CDShow::FindFilter(IGraphBuilder* pGraph,REFGUID iid,void** ppUnk)
{
	if(!pGraph||!ppUnk)
		return E_POINTER;

	HRESULT hr = E_FAIL;
	IEnumFilters* pEnum = NULL;
	IBaseFilter* pF = NULL;
	if(FAILED(pGraph->EnumFilters(&pEnum)))
		return hr;

	while(S_OK == pEnum->Next(1,&pF,0))
	{
		hr = pF->QueryInterface(iid,ppUnk);
		pF->Release();
		if(SUCCEEDED(hr))
			break;
	}
	pEnum->Release();

	return hr;
}

bool CDShow::IsVideoPinUnconnected(IBaseFilter* pBF)
{
	bool bRes = false;

	HRESULT hr = S_OK;
	IEnumPins* spPins = NULL;
	hr = pBF->EnumPins(&spPins);
	if(SUCCEEDED(hr))
	{
		ULONG ulFetched = 0;
		IPin* pPin = NULL;
		while(spPins->Next(1,&pPin,&ulFetched) == S_OK)
		{
			IPin* spPeerPin = NULL;
			hr = pPin->ConnectedTo(&spPeerPin);
			if(spPeerPin==NULL)
			{
				bRes = true;
				break;
			}

			pPin->Release();
			pPin = NULL;
		}
	}

	return bRes;
}

HRESULT CDShow::FindVideoOutPin(IBaseFilter* pBF,IPin** ppPin)
{
	HRESULT hr = S_OK;
	IEnumPins* spPins = NULL;
	hr = pBF->EnumPins(&spPins);
	if(SUCCEEDED(hr))
	{
		ULONG ulFetched = 0;
		IPin* pPin = NULL;
		while(spPins->Next(1,&pPin,&ulFetched) == S_OK)
		{
			AM_MEDIA_TYPE mt;
			hr = pPin->ConnectionMediaType(&mt);
			if(SUCCEEDED(hr)&&(mt.majortype==MEDIATYPE_Video))
			{
				hr = pPin->ConnectedTo(ppPin);
			}

			if(SUCCEEDED(hr))
				FreeMediaType(mt);
			pPin->Release();
		}
	}

	return hr;
}

HRESULT CDShow::FindConnectedPin(IBaseFilter* pBF,IPin** ppPin)
{
	HRESULT hr = S_OK;
	IEnumPins* spPins = NULL;
	hr = pBF->EnumPins(&spPins);
	if(SUCCEEDED(hr))
	{
		ULONG ulFetched = 0;
		IPin* pPin = NULL;
		if(spPins->Next(1,&pPin,&ulFetched) == S_OK)
		{
			hr = pPin->ConnectedTo(ppPin);

			pPin->Release();
		}
	}

	return hr;
}

int CDShow::GetPinCount(
    IBaseFilter *pBF, // Pointer to the starting filter
    PIN_DIRECTION Dir)    // Direction to search (upstream or downstream)
{
	int nCount = 0;

	HRESULT hr = S_OK;
	IEnumPins* spPins = NULL;
	hr = pBF->EnumPins(&spPins);
	if(SUCCEEDED(hr))
	{
		ULONG ulFetched = 0;
		IPin* pPin = NULL;
		while(spPins->Next(1,&pPin,&ulFetched) == S_OK)
		{
			PIN_DIRECTION eDir;
			hr = pPin->QueryDirection(&eDir);
			if(SUCCEEDED(hr)&&(eDir==Dir))
			{
				nCount++;
			}
			pPin->Release();
		}
	}

	return nCount;
}

HRESULT CDShow::GetPin(IBaseFilter *pBF,PIN_DIRECTION eDir,IPin** ppPin,bool bConnected)
{
	HRESULT hr = S_OK;
	IEnumPins* spPins = NULL;
	hr = pBF->EnumPins(&spPins);
	if(SUCCEEDED(hr))
	{
		ULONG ulFetched = 0;
		IPin* spPin = NULL;
		while(spPins->Next(1,&spPin,&ulFetched) == S_OK)
		{
			PIN_DIRECTION eTmpDir;
			hr = spPin->QueryDirection(&eTmpDir);
			if(SUCCEEDED(hr)&&(eTmpDir==eDir)&&(IsPinConnected(spPin)==bConnected))
			{
				*ppPin = spPin;
				(*ppPin)->AddRef();

				break;
			}
			
			spPin = NULL;
		}
	}

	return hr;
}

HRESULT CDShow::GetPin(IBaseFilter *pBF,PIN_DIRECTION eDir,IPin** ppPin)
{
	HRESULT hr = S_OK;
	IEnumPins* spPins = NULL;
	hr = pBF->EnumPins(&spPins);
	if(SUCCEEDED(hr))
	{
		ULONG ulFetched = 0;
		IPin* spPin = NULL;
		while(spPins->Next(1,&spPin,&ulFetched) == S_OK)
		{
			PIN_DIRECTION eTmpDir;
			hr = spPin->QueryDirection(&eTmpDir);
			if(SUCCEEDED(hr)&&(eTmpDir==eDir))
			{
				*ppPin = spPin;
				(*ppPin)->AddRef();

				break;
			}
			
			spPin = NULL;
		}
	}

	return hr;
}

IPin* CDShow::GetPin(IBaseFilter *pBF,PIN_DIRECTION eDir)
{
	IPin* spPin = NULL;
	GetPin(pBF,eDir,&spPin);
	return spPin;
}

bool CDShow::IsPinConnected(IPin* pPin)
{
	IPin* spPeerPin = NULL;
	pPin->ConnectedTo(&spPeerPin);
	return spPeerPin!=NULL;
}

//create Null Render
IPin* CDShow::CreateNullRender(IGraphBuilder * pGB)
{
	//Create Null Render
	IBaseFilter* spNullRender = NULL;
	HRESULT hr = AddFilterByClsid(pGB,CLSID_NullRenderer,NULL/*_T("Null Render")*/,&spNullRender);
	if(SUCCEEDED(hr))
	{
		return GetPin(spNullRender,PINDIR_INPUT);
	}

	return NULL;
}

HRESULT CDShow::ConnectAllPinToNullRender(IGraphBuilder * pGB,IBaseFilter *pBF)
{
	HRESULT hr = S_OK;
	IEnumPins* spPins = NULL;
	hr = pBF->EnumPins(&spPins);
	if(SUCCEEDED(hr))
	{
		ULONG ulFetched = 0;
		IPin* pPin = NULL;
		while(spPins->Next(1,&pPin,&ulFetched) == S_OK)
		{
			PIN_DIRECTION eDir;
			hr = pPin->QueryDirection(&eDir);
			if(SUCCEEDED(hr)&&(eDir==PINDIR_OUTPUT)&&!IsPinConnected(pPin))
			{
				IPin* spInPin = CreateNullRender(pGB);
				if(spInPin!=NULL)
					hr = pGB->ConnectDirect(pPin,spInPin,NULL);
			}
			pPin->Release();
		}
	}

	return hr;
}

HRESULT CDShow::RemoveFilter(IGraphBuilder * inGraph, IBaseFilter * inFilter)
{
	HRESULT hr = E_FAIL;

	if (inGraph && inFilter)
	{
		IEnumPins * pinEnum = 0;
		if (SUCCEEDED(inFilter->EnumPins(&pinEnum)))
		{
			pinEnum->Reset();
			IPin * pin = 0;
			ULONG cFetched = 0;
			bool pass = true;
			while (pass && SUCCEEDED(pinEnum->Next(1, &pin, &cFetched)))
			{
				if (pin && cFetched)
				{
					IPin * connectedPin = 0;
					pin->ConnectedTo(&connectedPin);
					if(connectedPin) 
					{
						inGraph->Disconnect(connectedPin);
						inGraph->Disconnect(pin);

						connectedPin->Release();
					}
					pin->Release();
				}
				else
				{
					pass = false;
				}
			}
			pinEnum->Release();

			hr = inGraph->RemoveFilter(inFilter);
		}
	}

	return hr;
}

//pPinʼλ˷ɾֻһPinFilterصһ
//1PinFilterPinppPinǸ
HRESULT CDShow::NukeUpstream(IGraphBuilder * inGraph, IBaseFilter * inFilter)
{
	HRESULT hr = E_FAIL;

	if (inGraph && inFilter)
	{
		IEnumPins * pinEnum = 0;
		if (SUCCEEDED(inFilter->EnumPins(&pinEnum)))
		{
			pinEnum->Reset();
			IPin * pin = 0;
			ULONG cFetched = 0;
			bool pass = true;
			while (pass && SUCCEEDED(pinEnum->Next(1, &pin, &cFetched)))
			{
				if (pin && cFetched)
				{
					IPin * connectedPin = 0;
					pin->ConnectedTo(&connectedPin);
					if(connectedPin) 
					{
						PIN_INFO pininfo;
						if (SUCCEEDED(connectedPin->QueryPinInfo(&pininfo)))
						{
							if(pininfo.dir == PINDIR_OUTPUT) 
							{
								int nPinCount = GetPinCount(pininfo.pFilter,PINDIR_OUTPUT);

								inGraph->Disconnect(connectedPin);
								inGraph->Disconnect(pin);

								//һPinɾ
								if(nPinCount==1)
								{
									hr = NukeUpstream(inGraph, pininfo.pFilter);
									inGraph->RemoveFilter(pininfo.pFilter);
								}
								//һPinδӵPinNull Render
								else if(nPinCount>1)
								{
									IPin* spInPin = CreateNullRender(inGraph);
									if(spInPin!=NULL)
										hr = inGraph->ConnectDirect(connectedPin,spInPin,NULL);
//									hr = ConnectAllPinToNullRender(inGraph,pininfo.pFilter,PINDIR_OUTPUT);
								}
							}
							pininfo.pFilter->Release();
						}
						connectedPin->Release();
					}
					pin->Release();
				}
				else
				{
					pass = false;
				}
			}
			pinEnum->Release();
		}
	}

	return hr;
}

//
HRESULT CDShow::GetAVRenderer(IGraphBuilder* pGB,CBaseFilterList& lstAudio,CBaseFilterList& lstVideo)
{
	HRESULT hr = S_OK;
	IEnumFilters* spEnumFilters = NULL;
	hr = pGB->EnumFilters(&spEnumFilters);
	if(SUCCEEDED(hr))
	{
		IBaseFilter* spBF = NULL;
		while(spEnumFilters->Next(1,&spBF,NULL)==S_OK)
		{
			//ΪǽҪ֪ǷIBasicVideoIBasicAudio
			//spFilterΪIUnknown
			IUnknown* spFilter = NULL;
			if(SUCCEEDED(spBF->QueryInterface(IID_IBasicVideo,(void**)&spFilter)))
			{
				lstVideo.push_back(spBF);
			}
			spFilter = NULL;
			if(SUCCEEDED(spBF->QueryInterface(IID_IBasicAudio,(void**)&spFilter)))
			{
				lstAudio.push_back(spBF);
			}
			
			spBF = NULL;
		}
	}

	return hr;
}

//  Free an existing media type (ie free resources it holds)

void WINAPI CDShow::FreeMediaType(AM_MEDIA_TYPE& mt)
{
    if (mt.cbFormat != 0) {
        CoTaskMemFree((PVOID)mt.pbFormat);

        // Strictly unnecessary but tidier
        mt.cbFormat = 0;
        mt.pbFormat = NULL;
    }
    if (mt.pUnk != NULL) {
        mt.pUnk->Release();
        mt.pUnk = NULL;
    }
}

// general purpose function to delete a heap allocated AM_MEDIA_TYPE structure
// which is useful when calling IEnumMediaTypes::Next as the interface
// implementation allocates the structures which you must later delete
// the format block may also be a pointer to an interface to release

void WINAPI CDShow::DeleteMediaType(AM_MEDIA_TYPE *pmt)
{
    // allow NULL pointers for coding simplicity

    if (pmt == NULL) {
        return;
    }

    FreeMediaType(*pmt);
    CoTaskMemFree((PVOID)pmt);
}

// ͨclsidʵFilter뵽GraphBuilder
HRESULT CDShow::AddFilterByClsid(IGraphBuilder* pGB, REFCLSID clsid,LPCWSTR lpszName,IBaseFilter** ppFilter)
{
	//Create Filter
	IBaseFilter* spFilter = NULL;
	HRESULT hr;

  // Create the filter 
  hr = CoCreateInstance(clsid, NULL, CLSCTX_INPROC, 
                        IID_IBaseFilter, (void **)&spFilter);
	if(SUCCEEDED(hr))
	{
		//add null render to graph builder
		hr = pGB->AddFilter(spFilter,lpszName);
		if(SUCCEEDED(hr)&&(ppFilter!=NULL))
		{
			*ppFilter = spFilter;
			(*ppFilter)->AddRef();
		}
	}

	return hr;
}

/*void CDShow::EnumDXSystemDevice(CLSID clsidType,PENUMDXSYSDEVICE pFn,void* pData)
{
	HRESULT hr = S_OK;
	ICreateDevEnum* spEnum = NULL;
 
	hr = CoCreateInstance(CLSID_SystemDeviceEnum,NULL,CLSCTX_INPROC,
												  IID_ICreateDevEnum,(void **)&spEnum);
	HANDLE_HR_EXP(hr);
	IEnumMoniker* spEnumMoniker = NULL;
	hr = spEnum->CreateClassEnumerator(clsidType,&spEnumMoniker,0);
	HANDLE_HR_EXP(hr);
	if(spEnumMoniker!=NULL)
	{
		IMoniker* pMoniker = NULL;
		ULONG ulFetched = 0;
		UINT index = 0;
		while(spEnumMoniker->Next(1,&pMoniker,&ulFetched) == S_OK)
		{
			bool bContinue = pFn(pMoniker,pData);
			pMoniker->Release();
			pMoniker = NULL;
			if(!bContinue)
				break;
		}
	}
}
*/
bool CDShow::OnEnumDXObjList(IMoniker* pMoniker,void* pData)
{
	CDX_ObjList& list = *(CDX_ObjList*)pData;
	IPropertyBag* spBag = NULL;
	HRESULT hr = pMoniker->BindToStorage(0,0,IID_IPropertyBag,(void**)&spBag);
	if(SUCCEEDED(hr))
	{
		DX_OBJ DxObj;
		VARIANT var;
		var.vt = VT_BSTR;
		hr = spBag->Read(L"FriendlyName",&var,0);
		if(SUCCEEDED(hr))
		{
			DxObj.strFriendName = var.bstrVal;
			::SysFreeString(var.bstrVal);
		}
		var.vt = VT_BSTR;
		hr = spBag->Read(L"CLSID",&var,0);
		if(SUCCEEDED(hr))
		{
			DxObj.strDispName = var.bstrVal;
			::SysFreeString(var.bstrVal);
		}

		list[UINT(list.size())] = DxObj;
	}

	return true;
}

void CDShow::GetDXObjList(CLSID clsidType,CDX_ObjList& list)
{
	list.clear();
//	EnumDXSystemDevice(clsidType,&OnEnumDXObjList,&list);
}

bool CDShow::OnEnumFindSysDev(IMoniker* pMoniker,void* pData)
{
	LPCTSTR lpszName = LPCTSTR(pData);
	IMoniker** ppmk = (IMoniker**)(pData);
	ppmk++;
	
	IPropertyBag* spBag = NULL;
	HRESULT hr = pMoniker->BindToStorage(0,0,IID_IPropertyBag,(void**)&spBag);
	if(SUCCEEDED(hr))
	{
		DX_OBJ DxObj;
		VARIANT var;
		var.vt = VT_BSTR;
		hr = spBag->Read(L"CLSID",&var,0);
		if(SUCCEEDED(hr))
		{
			DxObj.strDispName = var.bstrVal;
			::SysFreeString(var.bstrVal);
			if((DxObj.strDispName==lpszName)&&(*ppmk==NULL))
			{
				*ppmk = pMoniker;
				(*ppmk)->AddRef();
			}
		}
	}

	return true;
}

HRESULT CDShow::FindSysDev(CLSID clsidType,LPCTSTR lpszName,IMoniker** ppmk)
{
	*ppmk = NULL;
	void* pData[]={(void*)lpszName,ppmk};
//	EnumDXSystemDevice(clsidType,&OnEnumFindSysDev,pData);
	
	return S_OK;
}

HRESULT CDShow::CreateFilterByName(LPCTSTR lpszName,IBaseFilter** ppFilter)
{
	IBindCtx* pCtx;
	HRESULT hr = ::CreateBindCtx(0,&pCtx);
	ULONG ulEaten = 0;
	IMoniker* pmk = NULL;
	
	WCHAR Name[MAX_PATH];
	wsprintfW(Name, L"%hs",lpszName	);

	hr = ::MkParseDisplayName(pCtx,(LPCOLESTR)Name,&ulEaten,&pmk);
	pCtx->Release();
	if(SUCCEEDED(hr))
	{
		hr = pmk->BindToObject(NULL,NULL,IID_IBaseFilter,(void**)ppFilter);
		pmk->Release();
	}

	return hr;
}

// ͨdisplaynameʵFilter뵽GraphBuilder
HRESULT CDShow::AddFilterByName(IGraphBuilder* pGB, LPCTSTR lpszDispName,LPCTSTR lpszName,IBaseFilter** ppFilter)
{
	//Create Filter
	IBaseFilter* spFilter = NULL;
	HRESULT hr = CreateFilterByName(lpszDispName,&spFilter);
	if(SUCCEEDED(hr))
	{
		hr = pGB->AddFilter(spFilter,(LPCWSTR)lpszName);
		if(SUCCEEDED(hr)&&(ppFilter!=NULL))
		{
			*ppFilter = spFilter;
			(*ppFilter)->AddRef();
		}
	}

	return hr;
}

bool CDShow::OnEnumGetDispNameByIndex(IMoniker* pMoniker,void* pData)
{
	bool bRes = true;
	LONGLONG* pTmpData = (LONGLONG*)pData;
	
	//ҵ
	if(pTmpData[1]==pTmpData[0])
	{
		LPWSTR wszDisplayName = NULL;
		HRESULT hr = pMoniker->GetDisplayName(NULL,NULL,&wszDisplayName);
		if(SUCCEEDED(hr))
		{
			*(CString*)pTmpData[2] = wszDisplayName;
			CoTaskMemFree(wszDisplayName);
			bRes = false;
		}
	}
	pTmpData[1]++;

	return bRes;
}

// ɸIndexضӦƵƵѹDispName
CString CDShow::GetDispNameByIndex(bool bAudio,int nIndex)
{
	HRESULT hr = S_OK;
	CString str;
	try
	{
		LONGLONG Data[3] = {nIndex,0,LONGLONG(&str)};

/*		CDShow::EnumDXSystemDevice(
			bAudio?CLSID_AudioCompressorCategory:CLSID_VideoCompressorCategory,
			&OnEnumGetDispNameByIndex,
			&Data);
		*/
	}
	catch(...)
	{
		hr = E_FAIL;
	}
	return str;
}

bool CDShow::OnEnumListCompressors(IMoniker* pMoniker,void* pData)
{
	int& nIndex = *(int*)pData;
	
	IPropertyBag* spBag = NULL;
	HRESULT hr = pMoniker->BindToStorage(0,0,IID_IPropertyBag,(void**)&spBag);
	if(SUCCEEDED(hr))
	{
		DX_OBJ DxObj;
		VARIANT var;
		var.vt = VT_BSTR;
		hr = spBag->Read(L"FriendlyName",&var,0);
		if(SUCCEEDED(hr))
		{
			USES_CONVERSION;
			CString str = var.bstrVal;
//			printf("[%2d] %s\n",nIndex++,T2A(str));
			::SysFreeString(var.bstrVal);
		}
	}

	return true;
}

// гϵͳаװƵѹ
HRESULT CDShow::ListCompressors(bool bAudio)
{
	HRESULT hr = S_OK;
	try
	{
		int nIndex = 0;
/*		CDShow::EnumDXSystemDevice(
			bAudio?CLSID_AudioCompressorCategory:CLSID_VideoCompressorCategory,
			&OnEnumListCompressors,
			&nIndex);
			*/
	}
	catch(...)
	{
		hr = E_FAIL;
	}
	return hr;
}

