//-----------------------------------------------------------------------------
// Desc: Initilizes DirectSound mit Standardtreibern zu Beginn
//-----------------------------------------------------------------------------
HRESULT CSoundsApp::InitDirectSoundInterfaces()
{
	HRESULT hr;

  // Release DirectSound interfaces
	FreeDirectSound();

  // Create IDirectSound using the select sound device
  if( FAILED( hr = DirectSoundCreate8( m_pSoundGUID, &m_pDS, NULL ) ) )
		{
		  AfxMessageBox(E_INITDIRECTSOUNDINTERFACES_DIRECTSOUNDCREATE8);
			return S_FALSE;
		}

  // Create IDirectSoundCapture using the select capture device
  if( FAILED( hr = DirectSoundCaptureCreate8( m_pCaptureGUID, &m_pDSCapture, NULL ) ) )
		{
		  AfxMessageBox(E_INITDIRECTSOUNDINTERFACES_DIRECTSOUNDCAPTURECREATE8);
			return S_FALSE;
		}

  // Set DirectSound coop level 
  if( FAILED(m_pDS->SetCooperativeLevel( AfxGetMainWnd()->GetSafeHwnd(), DSSCL_PRIORITY ) ) )
		{
		  AfxMessageBox(E_INITDIRECTSOUNDINTERFACES_SETCOOPERATIVELEVEL);
			return S_FALSE;
		}

	// Set primary buffer format to the best supported Format
    WAVEFORMATEX  wfx;
    INT iIndex=0,Highest=0;
		// Try 20 different formats to see if they are supported
    for( iIndex = 0; iIndex < 20; iIndex++ )
    {
        mp_doc->GetWaveFormatFromIndex( iIndex, &wfx );

        if( SetPrimaryBufferFormat( wfx.nChannels, wfx.nSamplesPerSec, wfx.wBitsPerSample )==S_OK ) 
            Highest= iIndex;

        SAFE_RELEASE(m_pDSBPrimary);
    }
  mp_doc->GetWaveFormatFromIndex( Highest, &wfx );

	if( FAILED( hr = SetPrimaryBufferFormat( wfx.nChannels, wfx.nSamplesPerSec, wfx.wBitsPerSample ) ) )
		{
		  AfxMessageBox(E_INITDIRECTSOUNDINTERFACES_SETPRIMARAYBUFFERFORMAT);
			return S_FALSE;
		}

	//Erstelle Soundmanagerobjekt fr Zugriff auf Funktionen
	m_pSoundManager = new CSoundManager();

	//Dem Soundmanagerobjekt die Schnittstelle zuweisen
	m_pSoundManager->m_pDS=m_pDS;

/*
	WAVEFORMATEX* pwfx=new WAVEFORMATEX;
	pwfx->wFormatTag=WAVE_FORMAT_PCM;
	pwfx->nChannels=2;
	pwfx->nSamplesPerSec=96000;
	pwfx->nAvgBytesPerSec=96000*2;
	pwfx->nBlockAlign=2;
	pwfx->wBitsPerSample=8;

	DSBUFFERDESC dsbd;
  ZeroMemory( &dsbd, sizeof(DSBUFFERDESC) );
  dsbd.dwSize          = sizeof(DSBUFFERDESC);
  dsbd.dwFlags         = DSBCAPS_CTRLVOLUME;
  dsbd.dwBufferBytes   = 96000*2;
  dsbd.lpwfxFormat     = pwfx;

  if( FAILED( hr = m_pDS->CreateSoundBuffer( &dsbd, &m_pTonBuffer, NULL ) ) )
		AfxMessageBox("CreateSoundBuffer");
*/

  return S_OK;
}

//-----------------------------------------------------------------------------
// Desc: Releases DirectSound 
//-----------------------------------------------------------------------------
HRESULT CSoundsApp::FreeDirectSound()
{
  // Release DirectSound interfaces
//  SAFE_RELEASE(m_pTonBuffer);
  SAFE_RELEASE(m_pDSBPrimary);
	SAFE_RELEASE(m_pDSCapture);
	SAFE_DELETE(m_pSoundManager); //m_pDS wird im SoundmanagerDestruktor mit abgebaut

  return S_OK;
}

//-----------------------------------------------------------------------------
// Name: CSoundsApp::SetPrimaryBufferFormat()
// Desc: Set primary buffer to a specified format 
//-----------------------------------------------------------------------------
HRESULT CSoundsApp::SetPrimaryBufferFormat( DWORD dwPrimaryChannels, 
                                               DWORD dwPrimaryFreq, 
                                               DWORD dwPrimaryBitRate )
{
    HRESULT             hr;

    if( m_pDS == NULL )
        return CO_E_NOTINITIALIZED;

    // Get the primary buffer 
    DSBUFFERDESC dsbd;
    ZeroMemory( &dsbd, sizeof(DSBUFFERDESC) );
    dsbd.dwSize        = sizeof(DSBUFFERDESC);
    dsbd.dwFlags       = DSBCAPS_PRIMARYBUFFER;
    dsbd.dwBufferBytes = 0;
    dsbd.lpwfxFormat   = NULL;
       
    if( FAILED( hr = m_pDS->CreateSoundBuffer( &dsbd, &m_pDSBPrimary, NULL ) ) )
		{
			return S_FALSE;
		}

    WAVEFORMATEX wfx;
    ZeroMemory( &wfx, sizeof(WAVEFORMATEX) ); 
    wfx.wFormatTag      = WAVE_FORMAT_PCM; 
    wfx.nChannels       = (WORD) dwPrimaryChannels; 
    wfx.nSamplesPerSec  = dwPrimaryFreq; 
    wfx.wBitsPerSample  = (WORD) dwPrimaryBitRate; 
    wfx.nBlockAlign     = (WORD) (wfx.wBitsPerSample / 8 * wfx.nChannels);
    wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;

    if( FAILED( hr = m_pDSBPrimary->SetFormat(&wfx) ) )
		{
			return S_FALSE;
		}

    return S_OK;
}


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// Dialog Soundkarte einstellen aufgerufen                                     !
//                                                                             !
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

HRESULT CSoundsApp::OnEinstellungenSoundkarte() 
{
  CSoundkarteDlg SoundDlg;
	if (SoundDlg.DoModal() == IDOK)
	{
		m_pSoundGUID=SoundDlg.pSoundGUID;
		m_pCaptureGUID=SoundDlg.pCaptureGUID;
		m_strSoundEingang=SoundDlg.strSoundEingang;
		m_strSoundAusgang=SoundDlg.strSoundAusgang;
		mp_doc->UpdateAllViews(NULL,50);
		if( FAILED( InitDirectSoundInterfaces() ) )
		{
		  AfxMessageBox(E_ONEINSTELLUNGENSOUNDKARTE_INITDIRECTSOUNDINTERFACES);
			FreeDirectSound();
			return S_FALSE;
		}
		return S_OK;
	}
	else
		return S_FALSE;
}
