#if !defined(DIVISIONS_INCLUDED)
#define DIVISIONS_INCLUDED

enum divisions {linear, reciprocal, exponential};

class division_range;

struct simple_division_range {
	float min, max;
	long n;

	void write (FILE *file) const {
//		fprintf(file,"%f  %f  %ld\n",min,max,n);
	}
	void read (FILE *file) {
		fscanf(file,"%f  %f  %ld\n",&min,&max,&n);
	}
	division_range* new_division (divisions div); //const;
};


class division_range {
	protected:
	float lower, upper;
	long num;

	public:
	void init(const simple_division_range &rng) {
		lower = rng.min;
		upper = rng.max;
		num   = rng.n;
	}
	virtual float get_nth(long n) = 0;
	virtual void write_gp_scale (FILE *file) {};
};

class division_linear:public division_range {
	public:
	virtual float get_nth(long n) {
		return lower+float(n)/float(num)*(upper-lower);
	};
};

class division_reciprocal:public division_range {
	public:
	virtual float get_nth(long n) {
		float ur = 1.0/upper, lr = 1.0/lower;
		return 1.0/(lr+float(n)/float(num)*(ur-lr));
	};
};

class division_exponential:public division_range {
	public:
	virtual float get_nth(long n) {
		return lower*pow((upper/lower),float(n)/float(num));
	};
	virtual void write_gp_scale (FILE *file) {
//		fprintf (file, "set logscale x\n");
	};
};

division_range* simple_division_range::new_division (divisions div) //const 
{
	division_range *dr = NULL;
	switch (div) {
	case linear:
		dr = new division_linear();
		break;
	case reciprocal:
		dr = new division_reciprocal();
		break;
	case exponential:
		dr = new division_exponential();
		break;
	}
	dr->init (*this);
	return dr;
}

#endif //DIVISIONS_INCLUDED
