// WaveletTrafo.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "Sounds.h"
#include "WaveletTrafo.h"
#include "cwt.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWaveletTrafo


CWaveletTrafo::CWaveletTrafo()
{	
	m_arguments.signal  = NULL;
	m_arguments.wavelet = "haar";		/*haar, "Haar wavelet", {Haar-Wavelet}
																	  mexicanhat, "Mexican hat", {Mexikanischer Hut}
	Beispiele:												sinus, "Sine wave", {Sinuswelle}
	m_pcWavelet = "haar";							poisson, "Poisson wavelet", {Poisson-Wavelet}
	m_pcWavelet = "mexicanhat";				cauchy, "Cauchy wavelet (complex); argument: order", 
	m_pcWavelet = "sinus";						{Cauchy-Wavelet beliebiger Ordnung}
	m_pcWavelet = "poisson";					spinning, "Variant of the Cauchy wavelet needed for 
	m_pcWavelet = "cauchy,2";					analysis of signals from spinning machines (complex); 
	m_pcWavelet = "cauchy,5";					argument: mu1, mu2", 
	m_pcWavelet = "morlet,1";					{Verallgemeinertes Cauchy-Wavelet fr die Analyse von Spinnrotordaten}
	m_pcWavelet = "morlet,2";					morlet, "Morlet wavelet (complex); argument: base scale", 
	m_pcWavelet = "bspline,2,2,2";		{Morlet-Wavelet mit unabh"angig w"ahlbarer Skale}
	m_pcWavelet = "bspline,10,10";		bspline, "Complex wave in a B-Spline envelope (complex); 
																		arguments: list of window widths for building the B-spline", 
																		{B-Spline-Wavelet}*/

	m_arguments.usesignalrate = FALSE;				/* Das Wavelet wird fur jede Skale neu diskretisiert so dass es zu der
																	Abtastung des Eingangssignals passt. Diese Methode ist fr diskrete 
																	Eingangssignale angeraten. Bei zu kleinen Skalen arbeitet 
																	das Verfahren nicht zufriedenstellend, was einfach daran liegt, 
																	dass das Signal nicht fein genug abgetastet wurde. Dieses Verfahren
																	muss mit der Option UsesSignalRate=TRUE angewahlt werden.*/

	m_arguments.explicit_mi = false;					/*  Das Wavelet wird einmal mit einer festen Anzahl Sttzstellen diskretisiert 
																	und die Skalarprodukte werden mit skalierten Version des Signals berechnet. 
																	Diese Vorgehensweise empfiehlt sich bei kontinuierlichen Eingangssignalen
																	, weil nur kontinuierliche Funktionen beliebig genau	abgetastet 
																	werden knnen. Diese kommen aber im Programm nicht vor, da nur mit
																	Samples gearbeitet wird.*/
	m_arguments.mi   = 200;								//Anzahl der Sttzstellen fr Diskretisierung des Wavelets, falls es doch
																	//explizit angegeben werden will

	m_arguments.explicit_nb = TRUE;					//Translationen (Bereich der Zeitachse) 
	m_arguments.b.n = 20;									  // Feinheit der Unterteilungen des Zeitbereichs (quidstant)
																	// wird abhngig von bmin, bmax berechnet, falls nicht explizit vorgeben
	m_arguments.b.min = 0.0f;										//Zeitbereich der Waveletanzeige in Sekunden  1.Sample=Zeit 0
	m_arguments.b.max = 0.1f;

	m_arguments.explicit_adiv = TRUE;				//immer TRUE, den Skalenaufteilung wird explizit vorgegeben unter m_adiv
	m_arguments.adiv = exponential;								//Skalenaufteilung des Frequenzbereichs: 
																	//linear (wie FFT), reciprocal oder exponential (Oktaven)  
	m_arguments.a.n = 100;										//Anzahl der Skalen zwischen der grten und der kleinsten
	m_arguments.a.min = 0.05f;									// Kehrwert der kleinsten untersuchten Frequenz (20Hz)
	m_arguments.a.max = 0.00005f;										// Kehrwert der grten untersuchten Frequenz (20 kHz)


	m_arguments.samplerate = 44100;							//Samplingrate des Signals

/* Default values. 
	
	m_pcSignal  = NULL;
	m_strWavelet = "haar";		haar, "Haar wavelet", {Haar-Wavelet}
																	  mexicanhat, "Mexican hat", {Mexikanischer Hut}
	Beispiele:												sinus, "Sine wave", {Sinuswelle}
	m_pcWavelet = "haar";							poisson, "Poisson wavelet", {Poisson-Wavelet}
	m_pcWavelet = "mexicanhat";				cauchy, "Cauchy wavelet (complex); argument: order", 
	m_pcWavelet = "sinus";						{Cauchy-Wavelet beliebiger Ordnung}
	m_pcWavelet = "poisson";					spinning, "Variant of the Cauchy wavelet needed for 
	m_pcWavelet = "cauchy,2";					analysis of signals from spinning machines (complex); 
	m_pcWavelet = "cauchy,5";					argument: mu1, mu2", 
	m_pcWavelet = "morlet,1";					{Verallgemeinertes Cauchy-Wavelet fr die Analyse von Spinnrotordaten}
	m_pcWavelet = "morlet,2";					morlet, "Morlet wavelet (complex); argument: base scale", 
	m_pcWavelet = "bspline,2,2,2";		{Morlet-Wavelet mit unabh"angig w"ahlbarer Skale}
	m_pcWavelet = "bspline,10,10";		bspline, "Complex wave in a B-Spline envelope (complex); 
																		arguments: list of window widths for building the B-spline", 
																		{B-Spline-Wavelet}

	m_bUseSignalRate = FALSE;				 Das Wavelet wird fur jede Skale neu diskretisiert so dass es zu der
																	Abtastung des Eingangssignals passt. Diese Methode ist fr diskrete 
																	Eingangssignale angeraten. Bei zu kleinen Skalen arbeitet 
																	das Verfahren nicht zufriedenstellend, was einfach daran liegt, 
																	dass das Signal nicht fein genug abgetastet wurde. Dieses Verfahren
																	muss mit der Option UsesSignalRate=TRUE angewahlt werden.

	m_bExplicit_mi = false;					  Das Wavelet wird einmal mit einer festen Anzahl Sttzstellen diskretisiert 
																	und die Skalarprodukte werden mit skalierten Version des Signals berechnet. 
																	Diese Vorgehensweise empfiehlt sich bei kontinuierlichen Eingangssignalen
																	, weil nur kontinuierliche Funktionen beliebig genau	abgetastet 
																	werden knnen. Diese kommen aber im Programm nicht vor, da nur mit
																	Samples gearbeitet wird.
	m_longMi   = 200;								//Anzahl der Sttzstellen fr Diskretisierung des Wavelets, falls es doch
																	//explizit angegeben werden will

	m_bExplicit_nb = TRUE;					//Translationen (Bereich der Zeitachse) 
	m_b.n = 20;									  // Feinheit der Unterteilungen des Zeitbereichs (quidstant)
																	// wird abhngig von bmin, bmax berechnet, falls nicht explizit vorgeben
	m_b.min = 0.0f;										//Zeitbereich der Waveletanzeige in Sekunden  1.Sample=Zeit 0
	m_b.max = 0.1f;

	m_bExplicit_adiv = TRUE;				//immer TRUE, den Skalenaufteilung wird explizit vorgegeben unter m_adiv
	m_adiv = exponential;								//Skalenaufteilung des Frequenzbereichs: 
																	//linear (wie FFT), reciprocal oder exponential (Oktaven)  
	m_a.n = 100;										//Anzahl der Skalen zwischen der grten und der kleinsten
	m_a.min = 0.05f;									// Kehrwert der kleinsten untersuchten Frequenz (20Hz)
	m_a.max = 0.00005f;										// Kehrwert der grten untersuchten Frequenz (20 kHz)


	m_fSamplerate = 44100;							//Samplingrate des Signals

	*/
//	SetArguments();
}

void CWaveletTrafo::SetArguments()
{	
/*	m_arguments.a.n=m_a.n;
	m_arguments.a.max=m_a.max;
	m_arguments.a.min=m_a.min;
	m_arguments.b.n=m_b.n;
	m_arguments.b.max=m_b.max;
	m_arguments.b.min=m_b.min;
	m_arguments.adiv=m_adiv;
	m_arguments.bmaxint=m_longBMaxInt;
	m_arguments.bminint=m_longBMinInt;
	m_arguments.explicit_adiv=m_bExplicit_adiv;
	m_arguments.explicit_mi=m_bExplicit_mi;
	m_arguments.explicit_nb=m_bExplicit_nb;
	m_arguments.mi=m_longMi;
	m_arguments.samplerate=m_fSamplerate;
	m_arguments.signal=m_pcSignal;
	m_arguments.usesignalrate=m_bUseSignalRate;
*/
//	m_arguments.wavelet=m_strWavelet;//.GetBuffer(100);
}

CWaveletTrafo::~CWaveletTrafo()
{
}

bool CWaveletTrafo::is_prefix (const_string str, const_string prefix, const_string &end) {
	long len = strlen(prefix);
	if (strncmp(str,prefix,len)==0) {
		end = str+len;
		return true;
	} else {
		return false;
	}
}

void CWaveletTrafo::Start(SAMPLE* input,int input_length,CArray<FLOAT,FLOAT>* pPGMArray)
{
//	m_longBMinInt = long(floor(m_b.min * m_fSamplerate));
//	m_longBMaxInt = long(ceil (m_b.max * m_fSamplerate));
//	if (!m_bExplicit_nb) {
//		m_b.n = m_longBMaxInt - m_longBMinInt;
//	}
	m_arguments.bminint = long(floor(m_arguments.b.min * m_arguments.samplerate));
	m_arguments.bmaxint = long(ceil (m_arguments.b.max * m_arguments.samplerate));
	if (!m_arguments.explicit_nb) {
		m_arguments.b.n = m_arguments.bmaxint - m_arguments.bminint;
	}
//	SetArguments();

	char *wavelet_arguments = NULL;
	wavelet_fkt *wf=NULL;
	if(is_prefix(m_arguments.wavelet,"haar",wavelet_arguments)) {wf = new wavelet_haar();} else
	if(is_prefix(m_arguments.wavelet,"mexicanhat",wavelet_arguments)) {wf = new wavelet_mexicanhat();} else
	if(is_prefix(m_arguments.wavelet,"sinus",wavelet_arguments)) {wf = new wavelet_sinus();} else
	if(is_prefix(m_arguments.wavelet,"poisson",wavelet_arguments)) {wf = new wavelet_poisson();} else
	if(is_prefix(m_arguments.wavelet,"cauchy",wavelet_arguments)) {wf = new wavelet_cauchy();} else
	if(is_prefix(m_arguments.wavelet,"spinning",wavelet_arguments)) {wf = new wavelet_spinning();} else
	if(is_prefix(m_arguments.wavelet,"morlet",wavelet_arguments)) {wf = new wavelet_morlet();} else
	if(is_prefix(m_arguments.wavelet,"bspline",wavelet_arguments)) {wf = new wavelet_bspline();	}
	if (wf!=NULL) 
	{
		wf->init();
		wf->init_args(wavelet_arguments); 

	signal_fkt *sf=NULL;
	// Einlesen der Daten aus Array
	sf = new signal_discrete(input,input_length,m_arguments,wf);

//	division_range* adiv =	//m_arguments.a.new_division (m_arguments.adiv);
	division_range* adiv = NULL;
	if (m_arguments.adiv==linear) adiv = new division_linear();
	if (m_arguments.adiv==reciprocal) adiv = new division_reciprocal();
	if (m_arguments.adiv==exponential) adiv = new division_exponential();

	simple_division_range rng;
	rng.max=m_arguments.a.max;
	rng.min=m_arguments.a.min;
	rng.n=m_arguments.a.n;
	adiv->init (rng);

	const char *outputbasename="c:/Dumps/output";
	pPGMArray->RemoveAll();
	wf->transform (sf, adiv, outputbasename, m_arguments, pPGMArray);

	//after calculating the transform we determine whether we had enough input data
	//by checking the amplitude of the array
	//if everything is half of 8 Bit, there was not enough data and we delete the array
	//to display a hint message in CWaveletView
	int i=0;
	while (pPGMArray->GetAt(i)==127) 
	{
		if (i==pPGMArray->GetSize()-1) 
			break;
		i++;
	}
	if (i==pPGMArray->GetSize()-1) 
	{
		pPGMArray->RemoveAll();
	}
	else
	{
		pPGMArray->InsertAt(0,m_arguments.b.n);
		pPGMArray->InsertAt(0,m_arguments.a.n);
	}

	//DEBUGGING
/*	long nab = m_arguments.a.n*m_arguments.b.n;
	FILE* ioptr;
  ioptr = fopen("c://Dumps//ArrayPGM.txt","w");
	for(int i=0; i<nab+2; i++) {
		fprintf(ioptr, "%6ld: %f\n", i, pPGMArray->GetData()[i]);
	}
  fclose(ioptr);
*/
	wf->reset();
	delete adiv;
	delete sf;
	delete wf;
	}
} // end Start





